#include <stdio.h>
#include <string.h>
#include "defs.h"
#include "externs.h"
#include "protos.h"
#include "pce_inst.h"

unsigned char  header[512];	/* rom header */
unsigned char  tile_plane[128];	/* tile plane buffer */

/* ----
 * init_machine()
 * ----
 * specific machine initialization
 */

void
init_machine(void)
{
	addinst(pce_inst);
	addinst(pce_pseudo);
}

/* ----
 * write_header()
 * ----
 * generate and write rom header
 */

void
write_header(FILE *f, int banks)
{
	/* setup header */
	memset(header, 0, 512);
	header[0] = banks;

	/* write */
	fwrite(header, 512, 1, f);
}

/* ----
 * pack_8x8_tile()
 * ----
 * encode a 8x8 tile for the PC Engine
 */

int
pack_8x8_tile(void *data, int format, int offset)
{
	int i, j;
	int cnt;
	unsigned int   pixel;
	unsigned char *ptr;
	unsigned int  *packed;

	/* pack the tile only in the last pass */
	if (pass == FIRST_PASS)
		return (32);

	/* clear planes */
	for (i = 0; i < 32; i++)
		tile_plane[i] = 0;

	/* encode the tile */
	switch (format) {
	case CHUNKY_TILE:
		/* 8-bit chunky format */
		cnt = 0;
		ptr = data;

		for (i = 0; i < 8; i++) {
			for (j = 0; j < 8; j++) {
				pixel = ptr[j ^ 0x07];
				tile_plane[cnt]    |= (pixel & 0x01) ? (1 << j) : 0;
				tile_plane[cnt+1]  |= (pixel & 0x02) ? (1 << j) : 0;
				tile_plane[cnt+16] |= (pixel & 0x04) ? (1 << j) : 0;
				tile_plane[cnt+17] |= (pixel & 0x08) ? (1 << j) : 0;
			}				
			ptr += offset;
			cnt += 2;
		}
		break;

	case PACKED_TILE:
		/* 4-bit packed format */
		cnt = 0;
		packed = data;
	
		for (i = 0; i < 8; i++) {
			pixel = packed[i];
	
			for (j = 0; j < 8; j++) {
				tile_plane[cnt]    |= (pixel & 0x01) ? (1 << j) : 0;
				tile_plane[cnt+1]  |= (pixel & 0x02) ? (1 << j) : 0;
				tile_plane[cnt+16] |= (pixel & 0x04) ? (1 << j) : 0;
				tile_plane[cnt+17] |= (pixel & 0x08) ? (1 << j) : 0;
				pixel >>= 4;
			}
			cnt += 2;
		}
		break;
	}

	/* ok */
	return (32);
}

/* ----
 * pack_16x16_tile()
 * ----
 * encode a 16x16 tile (sprite format)
 */

int
pack_16x16_tile(void *data, int format, int offset)
{
	int i, j;
	int cnt;
	int pixel;
	unsigned char *ptr;
	unsigned int  *packed;

	/* pack the tile only in the last pass */
	if (pass == FIRST_PASS)
		return (128);

	/* clear planes */
	for (i = 0; i < 128; i++)
		tile_plane[i] = 0;

	/* encode the tile */
	switch (format) {
	case CHUNKY_TILE:
		/* 8-bit chunky format */
		cnt = 0;
		ptr = data;

		for (i = 0; i < 16; i++) {
			/* right column */
			for (j = 0; j < 8; j++) {
				pixel = ptr[j ^ 0x0F];
				tile_plane[cnt]    |= (pixel & 0x01) ? (1 << j) : 0;
				tile_plane[cnt+32] |= (pixel & 0x02) ? (1 << j) : 0;
				tile_plane[cnt+64] |= (pixel & 0x04) ? (1 << j) : 0;
				tile_plane[cnt+96] |= (pixel & 0x08) ? (1 << j) : 0;
			}

			/* left column */
			for (j = 0; j < 8; j++) {
				pixel = ptr[j ^ 0x07];
				tile_plane[cnt+1]  |= (pixel & 0x01) ? (1 << j) : 0;
				tile_plane[cnt+33] |= (pixel & 0x02) ? (1 << j) : 0;
				tile_plane[cnt+65] |= (pixel & 0x04) ? (1 << j) : 0;
				tile_plane[cnt+97] |= (pixel & 0x08) ? (1 << j) : 0;
			}				
			ptr += offset;
			cnt += 2;
		}
		break;

	case PACKED_TILE:
		/* 4-bit packed format */
		cnt = 0;
		packed = data;
	
		for (i = 0; i < 16; i++) {
			/* right column */
			pixel = packed[cnt];
	
			for (j = 0; j < 8; j++) {
				tile_plane[cnt]    |= (pixel & 0x01) ? (1 << j) : 0;
				tile_plane[cnt+32] |= (pixel & 0x02) ? (1 << j) : 0;
				tile_plane[cnt+64] |= (pixel & 0x04) ? (1 << j) : 0;
				tile_plane[cnt+96] |= (pixel & 0x08) ? (1 << j) : 0;
				pixel >>= 4;
			}

			/* left column */
			pixel = packed[cnt + 1];
	
			for (j = 0; j < 8; j++) {
				tile_plane[cnt]    |= (pixel & 0x01) ? (1 << j) : 0;
				tile_plane[cnt+33] |= (pixel & 0x02) ? (1 << j) : 0;
				tile_plane[cnt+65] |= (pixel & 0x04) ? (1 << j) : 0;
				tile_plane[cnt+97] |= (pixel & 0x08) ? (1 << j) : 0;
				pixel >>= 4;
			}
			cnt += 2;
		}
		break;
	}

	/* ok */
	return (128);
}

