#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "defs.h"
#include "externs.h"
#include "protos.h"

char   pcx_name[128];	/* PCX file name */
int    pcx_w, pcx_h;	/* PCX dimensions */
int    pcx_nb_colors;	/* number of colors (16/256) in the PCX */
unsigned char *pcx_buf;	/* pointer to the PCX buffer */
unsigned char  pcx_pal[256][3];	/* palette */
unsigned char  pcx_plane[128][4];
struct PCX_HEADER {		/* PCX file header */
	char  manufacturer, version;
	char  encoding;
	char  bpp;
	short xmin, ymin, xmax, ymax;
	short xdpi, ydpi;
	char  colormap[16][3];
	char  reserved;
	char  np;
	short bytes_per_line;
	short palette_info;
	short xscreen, yscreen;
	char  pad[54];
} pcx;

/* ----
 * check_args()
 * ----
 * check arguments
 */

int
check_args(int *arg, int n, int *a, int *b, int *c, int *d, int size)
{
	int x, y, w, h;

	x = 0;
	y = 0;

	if (n == 0) {		/* no arg */
		w = (pcx_w / size);
		h = (pcx_h / size);
	}
	else if (n == 2) {	/* 2 args */
		w = arg[0];
		h = arg[1];
	}
	else {				/* 4 args */
		x = arg[0];
		y = arg[1];
		w = arg[2];
		h = arg[3];
	}

	/* check */
	if (((x + w * size) > pcx_w) || ((y + h * size) > pcx_h)) {
		error("Coordinates out of range!");
		return (0);
	}

	/* write back the value */
	*a = x;
	*b = y;
	*c = w;
	*d = h;
	return (1);
}

/* ----
 * load_pcx()
 * ----
 * load a PCX file and unpack it
 */

int
load_pcx(char *name)
{
	FILE *f;

	/* check if the file is the same as the previously loaded one;
	 * if this is the case do not reload it
	 */
        #ifndef LINUX
	if (strlen(name) && (stricmp(pcx_name, name) == 0))
		return (1);
        #else
        if (strlen(name) && (strcmp(pcx_name, name) == 0))
                return (1);
        #endif
	else {
		/* no it's a new file - ok let's prepare loading */
		if (pcx_buf)
			free(pcx_buf);
		pcx_buf = NULL;
		pcx_name[0] = '\0';
	}

	/* open the file */
	if ((f = open_file(name, "rb")) == NULL) {
		error("Can not open file!");
		return (0);
	}

	/* get the picture size */
	fread(&pcx, 128, 1, f);
	pcx_w = (pcx.xmax - pcx.xmin + 1);
	pcx_h = (pcx.ymax - pcx.ymin + 1);

	/* adjust picture width */
	if (pcx_w & 0x01)
		pcx_w++;

	/* check size range */
	if ((pcx_w > 800) || (pcx_h > 600)) {
		error("Picture size too big, max. 800x600!");
		return (0);
	}
	if ((pcx_w < 16) || (pcx_h < 16)) {
		error("Picture size too small, min. 16x16!");
		return (0);
	}

	/* malloc a buffer */
    pcx_buf = malloc(pcx_w * pcx_h);
	if (pcx_buf == NULL) {
		error("Can not load file, not enough memory!");
		return (0);
	}

	/* decode the picture */
	if ((pcx.bpp == 8) && (pcx.np == 1))
		decode_256(f, pcx_w, pcx_h);
	else if ((pcx.bpp == 1) && (pcx.np <= 4))
		decode_16(f, pcx_w, pcx_h);
	else {
		error("Unsupported or invalid PCX format!");
		return (0);
	}

	fclose(f);
	strcpy(pcx_name, name);
	return (1);
}

/* ----
 * decode_256()
 * ----
 * decode a 256 colors PCX file
 */

void
decode_256(FILE *f, int w, int h)
{
	unsigned int   i, c, x, y;
	unsigned char *ptr;

	ptr = pcx_buf;
	x = 0;
	y = 0;

	/* decode */
	switch (pcx.encoding) {
	case 0:
		/* raw */
		fread(pcx_buf, w, h, f);
		c = fgetc(f);
		return;

	case 1:
		/* simple run-length encoding */
		do {
			c = fgetc(f);
			if (c == EOF)
				break;
			if ((c & 0xC0) != 0xC0)
				i = 1;
			else {
				i = (c & 0x3F);
				c = fgetc(f);
			}
			do {
				x++;
			   *ptr++ = c;
				if (x == w) {
					x = 0;
					y++;
				}
			} while (--i);
		} while (y < h);
		break;

	default:
		error("Unsupported PCX encoding scheme!");
		return;
	}			

	/* get the palette */
	if (c != EOF)
		c = fgetc(f);
	while ((c != 12) && (c != EOF))
		c = fgetc(f);
	if (c == 12)
		fread(pcx_pal, 768, 1, f);

	/* number of colors */
	pcx_nb_colors = 256;
}

/* ----
 * decode_16()
 * ----
 * decode a 16 (or less) colors PCX file
 */

void
decode_16(FILE *f, int w, int h)
{
	int i, j, k, n;
	int x, y, p;
	unsigned int c, pix;
	unsigned char *ptr;

	ptr = pcx_buf;
	x = 0;
	y = 0;
	p = 0;

	/* decode */
	switch (pcx.encoding) {
	case 0:
		/* raw */
		error("Unsupported PCX encoding scheme!");
		break;

	case 1:
		/* simple run-length encoding */
		do {
			/* get a char */
			c = fgetc(f);
			if (c == EOF)
				break;

			/* check if it's a repeat command */
			if ((c & 0xC0) != 0xC0)
				i = 1;
			else {
				i = (c & 0x3F);
				c = fgetc(f);
			}

			/* unpack */
			do {
			    pcx_plane[x >> 3][p] = c;
				x += 8;

				/* end of line */
				if (x >= w) {
					x = 0;
					p++;

					/* plane to chunky conversion */
					if (p == pcx.np) {
						p = 0;
						n = (w + 7) >> 3;
						y++;

						/* loop */
						for (j = 0; j < n; j++) {
							for (k = 7; k >= 0; k--) {
								/* get pixel index */
								pix = 0;

								switch (pcx.np) {
								case 4:
									pix |= ((pcx_plane[j][3] >> k) & 0x01) << 3;
								case 3:
									pix |= ((pcx_plane[j][2] >> k) & 0x01) << 2;
								case 2:
									pix |= ((pcx_plane[j][1] >> k) & 0x01) << 1;
								case 1:
									pix |= ((pcx_plane[j][0] >> k) & 0x01);
									break;
								}

								/* store pixel */
								if (x < w)
									*ptr++ = pix;

								x++;
							}
						}
						x = 0;
					}
				}
			}
			while (--i);
		}
		while (y < h);
		break;

	default:
		error("Unsupported PCX encoding scheme!");
		return;
	}			

	/* get the palette */
	memset(pcx_pal, 0, 768);
	memcpy(pcx_pal, pcx.colormap, (1 << pcx.np) * 3);

	/* number of colors */
	pcx_nb_colors = 16;
}

